#!/usr/bin/env python
'''A python module to generate lightcurve templates.
Author:  Chris Burns
Version:  0.1 (extreme-alpha)

This module uses Barry Madore's GLoEs algorithm to interpolate over a surface
with heterogeneous data.  In this case, we have a surface with x=time, y=dm15
and z=flux.

This module provides an object called template.  You create any number of these
and then simply ask it to generate a template of a given dm15 and then you can
evaluate that template at any time you like.  This is not a natural way to do
things, but it keeps it comaptible with the other dm15temp module I made for
Prieto's templates.

Example:
>>> import dm15temp2
>>> t = dm15temp2.template()
>>> t.mktemplate(1.6)
>>> flux,e_flux,mask = t.eval(arange(-10, 70, 1.0), 'B')

Here, we create an instance of the template, ask it to generate a lc template
with dm15 = 1.6, then we evaluate the flux on the interval [-10,70] in steps
of 1 day.  It returns a 3-tuple:  flux (normalized to peak of 1.0), error in
flux and a mask.  The mask is 1 where the template is well-behaved (you could,
afterall ask for a template that goes to day 1000, but its mask value will be 0,
because we have no constraining data there).

This module can also be run as a standalone script:

   ./dm15temp2 [dm15] [filter]

in which case, the template is sent to starndard out.
'''
import sys,os,string
import Numeric as num
import pickle

form = "%5.1lf %10.3lf %10.6lf %10.3lf %10.6lf %10.3lf %10.6lf %10.3lf %10.6lf"

class template:
   def __init__(self):
      self.dm15s = None
      #/* Reading data of the templates */
      self.dm15 = None
      self.Bsurf = pickle.load(open('B_surface.obj'))
      self.Vsurf = pickle.load(open('V_surface.obj'))
      self.usurf = pickle.load(open('u_s_surface.obj'))
      self.gsurf = pickle.load(open('g_s_surface.obj'))
      self.rsurf = pickle.load(open('r_s_surface.obj'))
      self.isurf = pickle.load(open('i_s_surface.obj'))
      self.t = None
      self.B = None;  self.eB = None
      self.V = None;  self.eV = None
      self.u = None;  self.eu = None
      self.g = None;  self.eg = None
      self.r = None;  self.er = None
      self.i = None;  self.ei = None

      self.sigmaxs = {'B':5.0, 'V':5.0, 'u':5.0, 'g':5.0, 'r':5.0, 'i':5.0}
      self.sigmays = {'B':0.34, 'V':0.34, 'u':0.34, 'g':0.34, 'r':0.34, 'i':0.34}

   def mktemplate(self, dm15, generate=1):
      '''2nd and 3rd arguments ignored.'''
      self.dm15 = dm15
      if generate:
         # generate the model light-curve.
         self.t = num.arange(-10,81, 1.0)
         for band in ['B','V','u','g','r','i']:
            self.__dict__[band],self.__dict__['e'+band], mask = \
               self.eval(band, self.t)

   def sigmax(self, x, y):
      '''Make a value of sigmax based on the values of time and dm15.'''
      sigma = 3.0 + 0.1*num.absolute(x)
      return(num.where(num.greater(sigma, 10), 10, sigma))

   def sigmay(self, x, y):
      '''Make a value of sigmay based on the values of time and dm15.'''

      return x*0 + 0.3

   def eval(self, band, times, z=0, sigmax=None, sigmay=None, mag=1, tol=0):
      if len(num.shape(times)) == 0:
         evt = num.array([times/(1+z)])
         scalar = 1
      else:
         evt = times/(1+z)
         scalar = 0
      if band not in self.__dict__:
         raise AttributeError, "Sorry, band %s is not supported by dm15temp" % \
               band
      if sigmax is None:
         sigmax = self.sigmax(evt, evt*0+self.dm15)
      if sigmay is None:
         sigmay = self.sigmay(evt, evt*0+self.dm15)

      flux,eflux = self.__dict__[band+'surf'].eval(evt, evt*0+self.dm15, 
            sigmax=sigmax, sigmay=sigmay, tol=tol)
      flux0,eflux0 = self.__dict__[band+'surf'].eval(0.0, self.dm15,
            sigmax=sigmax, sigmay=sigmay, tol=tol)

      # Now mask out [-10,80]
      gids = num.greater_equal(evt, -10)*num.less_equal(evt,70)
      if not mag:
         return(flux/flux0, eflux, gids)
      zids = num.greater(flux, 0)
      flux = num.where(zids, flux, 1.0)
      evd = -2.5*num.log10(flux/flux0)
      eevd = eflux/flux*1.0857
      gids = gids*zids
      if scalar:
         return(evd[0], eevd[0], gids[0])
      else:
         return(evd, eevd, gids)

if __name__ == "__main__":
   if len(sys.argv) < 3:
      print "Usage:  dm15temp2 dm15 filter"
      print "        dm15 = decline rate"
      print "        filter = one of u,g,r,i,B,V"
      sys.exit(1)

   dm15 = float(sys.argv[1])
   filter = sys.argv[2]
   t = template()
   t.mktemplate(dm15, generate=0)
   print "# This is dm15temp2 v. 0.1, using GLoEs with built-in weights"
   print "# Constructed light curve template for dm15=%.3f mag, filter=%s" % (dm15,filter)
   print "# Columns:  \n#  1    t(Bmax) \n#  2-3  M-M(max)   sigma[M-M(max)]"
   ts = num.arange(-10, 70, 1.0)
   m,em,mask = t.eval(filter, ts)
   for i in range(len(m)):
      if mask[i] == 1:
         print ts[i], m[i], em[i]
